from __future__ import annotations

import copy
from typing import TYPE_CHECKING, Any, Literal

from django.db.models import Q
from django.db.models.expressions import BaseExpression, Combinable
from django.http.request import QueryDict

from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.fields import SkipField, empty

from rest_filters.utils import AnyField, fill_q_template

if TYPE_CHECKING:
    from rest_framework.fields import _Empty

    from rest_filters.filtersets import FilterSet


class Entry:
    """
    Each Filter ultimately resolves to an Entry object, which contains
    information on how to modify a QuerySet.

    Attributes of Entry objects within the same group are combined to create a
    new Entry instance, representing the cumulative changes to the QuerySet
    made by all filters in that group.
    """

    def __init__(
        self,
        *,
        group: str = "chain",
        aliases: dict[str, Any] | None = None,
        value: Any,
        expression: Any,
    ):
        """
        :param group: Group of this Entry.
        :param aliases: Annotations that are going to be applied to QuerySet.
         These annotations can be referenced in the expression.
        :param value: Parsed value of the query parameter. If groups combined
         multiple query parameters, this will be a dictionary of
         ``{name: value}`` pairs.
        :param expression: Resolved query expression of this Entry.
        """
        self.group = group
        self.aliases = aliases
        self.value = value
        self.expression = expression

    def __repr__(self) -> str:
        return "Entry(group=%r, aliases=%r, value=%r, expression=%r)" % (
            self.group,
            self.aliases,
            self.value,
            self.expression,
        )

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, Entry):
            return NotImplemented
        return (
            self.group == other.group
            and self.aliases == other.aliases
            and self.value == other.value
            and self.expression == other.expression
        )


class Filter:
    def __init__(
        self,
        f: AnyField | None = None,
        /,
        *,
        field: str | BaseExpression | Combinable | None = None,
        lookup: str = "",
        template: Q | None = None,
        group: str | None = None,
        negate: bool = False,
        method: str | None = None,
        aliases: dict[str, Any] | None = None,
        param: str | None = None,
        children: list[Filter] | None = None,
        namespace: bool = False,
        blank: Literal["keep", "omit"] | None = None,
        noop: bool = False,
    ) -> None:
        """
        A data descriptor for query parameters.

        :param f: The serializer field that will parse this query parameter.
         This can be left empty if ``FilterSet.get_serializer`` is used to
         dynamically resolve the serializer field. A child filter may also omit
         this, in which case the parent serializer field is used.
        :param field: A string (field name) or Django expression that
         determines the database field/expression that is going to be filtered
         against. Child filters will inherit this value.
        :param lookup: The lookup that will be used with the resolved database
         field.
        :param template: A ``Q`` object that determines query expression for
         this parameter. For example, ``Q("username") | Q("email")`` will match
         the username or email field.
        :param group: Group identifier for this filter. Child filters will
         inherit their parent's group (if set).
        :param negate: Set this to ``True`` to negate the resolved query
         expression.
        :param method: A method on the current FilterSet class that is going to
         determine the query expression for this filter.
        :param aliases: Additional expressions that are going to be added to
         the QuerySet.
        :param param: The name of this query parameter.
        :param children: A list of filters that reside under the namespace of
         this filter.
        :param namespace: When enabled, marks this filter as *namespace* and
         disables it. Only the child filters belonging to this filter will be
         available.
        :param blank: Determines :py:attr:`rest_filters.conf.AppSettings.BLANK`
         behavior for this filter.
        :param noop: Set this to ``True`` to disable query expression
         resolution. In this mode, the query parameter won't do any filtering
         however, its value will be validated and available.
        """

        self._field = field
        self.lookup = lookup
        self.template = template

        if group is not None and not group.isidentifier():
            raise ValueError("Group names must be valid Python identifiers")
        if blank is not None and blank not in ("keep", "omit"):
            raise ValueError("blank must either be 'keep' or 'omit'")
        if template and lookup:
            raise ValueError(
                "'template' and 'lookup' cannot be used together. Add lookup to"
                " your template instead, for example: Q('username__icontains')"
            )
        if template and field:
            raise ValueError("'template' and 'field' cannot be used together")
        if negate and method:
            raise ValueError(
                "'method' and 'negate' cannot be used together. Negate the"
                " expression in your method instead."
            )

        self._group = group
        self.aliases = aliases

        self.negate = negate
        self.noop = noop

        self._blank = blank
        self.method = method
        self._param = param
        self._serializer = f
        self._filterset: FilterSet[Any] | None = None

        self.namespace = namespace
        self.parent: Filter | None = None
        self.children = children or []
        for child in self.children:
            child.bind(self)

        if not self.children and self.namespace:
            raise ValueError("Namespace filters are required to have child filters")

    def __repr__(self) -> str:
        args = []
        for name in (
            "_field",
            "lookup",
            "template",
            "_group",
            "aliases",
            "negate",
            "noop",
            "_blank",
            "method",
            "_param",
            "_serializer",
            "_filterset",
            "namespace",
            "children",
        ):
            attr = getattr(self, name)
            args.append("%s=%r" % (name, attr))
        return "%s(%s)" % (self.__class__.__name__, ", ".join(args))

    def __set_name__(self, owner: FilterSet[Any], name: str) -> None:
        self.name = name

    @property
    def blank(self) -> Literal["keep", "omit"]:
        if self._blank is None:
            filterset = self.get_filterset()
            return filterset.options.blank
        return self._blank

    def bind(self, parent: Filter) -> None:
        self.parent = parent

        self._param = self._param or self.lookup
        if not self._param:
            raise ValueError(
                "Either 'param' or 'lookup' parameter needs to"
                " be specified for child filters"
            )

    def get_group(self) -> str:
        if self._group is not None:
            return self._group
        elif self.parent is not None:
            return self.parent.get_group()
        return "chain"

    def get_db_field(self) -> str | BaseExpression | Combinable:
        if self.parent and self._field is None:
            return self.parent.get_db_field()
        return self._field or self.name

    def get_param_name(self) -> str:
        try:
            name = self._param or self.name
        except AttributeError as e:
            raise AssertionError("Could not resolve FilterSet") from e
        if self.parent is not None:
            namespace = self.parent.get_param_name()
            return f"{namespace}.{name}"
        return name

    def get_query_value(self, query_dict: QueryDict) -> str | _Empty:
        param = self.get_param_name()
        return query_dict.get(param, empty)

    def get_serializer(self) -> AnyField:
        if self._serializer is not None:
            return self._serializer
        elif self.parent is not None:
            try:
                return self.parent.get_serializer()
            except ValueError:
                pass
        raise ValueError(
            "Serializer could not be resolved for %r" % self.get_param_name()
        )

    def get_filterset(self) -> FilterSet[Any]:
        if self.parent:
            return self.parent.get_filterset()
        assert self._filterset is not None, "Could not resolve FilterSet"
        return self._filterset

    def resolve_serializer(self) -> AnyField:
        filterset, param = self.get_filterset(), self.get_param_name()
        try:
            serializer = self.get_serializer()
        except ValueError:
            dynamic = filterset.get_serializer(param, None)
            if dynamic is None:
                raise
            serializer = copy.deepcopy(dynamic)
        else:
            replacement = filterset.get_serializer(param, serializer)
            if replacement is None:
                raise ValueError(
                    "Serializer could not be resolved for %r" % self.get_param_name()
                )
            if replacement is not serializer:
                serializer = copy.deepcopy(replacement)

        serializer.default = filterset.get_default(param, serializer.default)
        serializer._context = filterset.get_serializer_context(param)  # type: ignore[attr-defined]
        return serializer

    def run_validation(self, value: str | _Empty, serializer: AnyField) -> Any:
        filterset, param = self.get_filterset(), self.get_param_name()
        return filterset.run_validation(value, serializer, param)

    def parse_value(self, value: str | _Empty) -> Any:
        if value is not empty:
            value = serializers.CharField(allow_blank=True).run_validation(value)
            if value == "" and self.blank == "omit":
                value = empty
        serializer = self.resolve_serializer()
        return self.run_validation(value, serializer)

    def resolve_entry_attrs(self, value: Any) -> Entry:
        if self.template is not None:
            template = ~self.template if self.negate else self.template
            expression = fill_q_template(template, value=value)
        else:
            lhs = self.get_db_field()
            if isinstance(lhs, (BaseExpression, Combinable)):
                alias = "_default_alias_%s" % self.get_param_name()
                if self.aliases is not None:
                    self.aliases[alias] = lhs
                else:
                    self.aliases = {alias: lhs}
                lhs = alias
            if self.lookup:
                lhs = f"{lhs}__{self.lookup}"
            expression = Q(**{lhs: value}, _negated=self.negate)
        return Entry(
            group=self.get_group(),
            aliases=self.aliases,
            value=value,
            expression=expression,
        )

    def resolve_entry(self, query_dict: QueryDict) -> Entry | None:
        try:
            value = self.parse_value(self.get_query_value(query_dict))
        except SkipField:
            return None
        if self.noop:
            return Entry(group=self.get_group(), value=value, expression=None)
        if self.method is not None:
            param = self.get_param_name()
            result = getattr(self.get_filterset(), self.method)(param, value)
            if isinstance(result, Entry):
                return result
            return Entry(group=self.get_group(), value=value, expression=result)
        return self.resolve_entry_attrs(value)

    def get_all_children(self) -> list[Filter]:
        children = []
        for child in self.children:
            children.append(child)
            children.extend(child.get_all_children())
        return children

    def resolve(
        self, query_dict: QueryDict
    ) -> tuple[dict[str, Entry | None], dict[str, Any]]:
        entries, errors = {}, {}
        for instance in [self, *self.get_all_children()]:
            if instance.namespace:
                continue
            param = instance.get_param_name()
            try:
                entry = instance.resolve_entry(query_dict)
            except ValidationError as err:
                errors[param] = err.detail
            else:
                entries[param] = entry
        return entries, errors



from __future__ import annotations

from collections import defaultdict
from collections.abc import Sequence
from typing import TYPE_CHECKING, Any

from django.utils.translation import gettext, ngettext

from rest_framework import serializers
from rest_framework.settings import api_settings

if TYPE_CHECKING:
    from django.utils.functional import _StrOrPromise as StrOrPromise

    from rest_filters.filtersets import FilterSet
else:
    from django.utils.functional import Promise as StrPromise

    StrOrPromise = str | StrPromise


__all__ = [
    "Constraint",
    "Dependency",
    "MethodConstraint",
    "MutuallyExclusive",
    "MutuallyInclusive",
]


class Constraint:
    def __init__(
        self,
        *,
        message: StrOrPromise = "",
        **kwargs: Any,
    ) -> None:
        self._message = message
        self.kwargs = kwargs
        self.filterset: FilterSet[Any] | None = None

    def get_message(self, values: dict[str, Any], **kwargs: Any) -> dict[str, Any]:
        message = self._message or gettext(
            "Request failed to meet constraint: %(constraint)s"
        ) % {"constraint": self.__class__.__name__}
        return {api_settings.NON_FIELD_ERRORS_KEY: [message]}

    def check(self, values: dict[str, Any]) -> None:
        raise NotImplementedError


class MethodConstraint(Constraint):
    def __init__(
        self,
        *,
        message: StrOrPromise = "",
        method: str,
        **kwargs: Any,
    ) -> None:
        self.method = method
        super().__init__(message=message, **kwargs)

    def check(self, values: dict[str, Any]) -> None:
        assert self.filterset, "Missing filterset for constraint"
        getattr(self.filterset, self.method)(values)


class MutuallyExclusive(Constraint):
    def __init__(
        self,
        *,
        message: StrOrPromise = "",
        fields: Sequence[str],
        **kwargs: Any,
    ) -> None:
        assert len(fields) > 1, "Provide 2 or more fields for this constraint"
        self.fields = fields
        super().__init__(message=message, **kwargs)

    def get_message(self, values: dict[str, Any], **kwargs: Any) -> dict[str, Any]:
        if self._message:
            return super().get_message(values, **kwargs)
        return {
            api_settings.NON_FIELD_ERRORS_KEY: [
                gettext(
                    "The following fields are mutually exclusive,"
                    " you may only provide one of them: %(fields)s"
                )
                % {
                    "fields": ", ".join(
                        f'"{field}"' for field in self.fields if field in values
                    )
                }
            ]
        }

    def check(self, values: dict[str, Any]) -> None:
        if sum(field in values for field in self.fields) > 1:
            raise serializers.ValidationError(self.get_message(values))


class MutuallyInclusive(Constraint):
    def __init__(
        self,
        *,
        message: StrOrPromise = "",
        fields: Sequence[str],
        **kwargs: Any,
    ) -> None:
        assert len(fields) > 1, "Provide 2 or more fields for this constraint"
        self.fields = fields
        super().__init__(message=message, **kwargs)

    def get_message(self, values: dict[str, Any], **kwargs: Any) -> dict[str, Any]:
        if self._message:
            return super().get_message(values, **kwargs)
        return {
            api_settings.NON_FIELD_ERRORS_KEY: [
                gettext(
                    "The following fields are mutually inclusive,"
                    " you must provide them all at once or none of them: %(fields)s"
                )
                % {"fields": ", ".join(f'"{field}"' for field in self.fields)}
            ]
        }

    def check(self, values: dict[str, Any]) -> None:
        fields = [field in values for field in self.fields]
        if any(fields) and not all(fields):
            raise serializers.ValidationError(self.get_message(values))


class Dependency(Constraint):
    def __init__(
        self,
        *,
        message: StrOrPromise = "",
        fields: Sequence[str],
        depends_on: Sequence[str],
        **kwargs: Any,
    ) -> None:
        self.fields = fields
        self.depends_on = depends_on
        super().__init__(message=message, **kwargs)

    def get_message(self, values: dict[str, Any], **kwargs: Any) -> dict[str, Any]:
        if self._message:
            return super().get_message(values, **kwargs)
        missing = kwargs["missing"]
        message = {}
        for field, dependencies in missing.items():
            message[field] = [
                ngettext(
                    "This query parameter also requires the following"
                    " parameter to be present: %(params)s",
                    "This query parameter also requires the following"
                    " parameters to be present: %(params)s",
                    len(dependencies),
                )
                % {"params": ", ".join(f'"{field}"' for field in dependencies)}
            ]
        return message

    def check(self, values: dict[str, Any]) -> None:
        missing = defaultdict(list)
        for field in self.fields:
            for dependency in self.depends_on:
                if field in values and dependency not in values:
                    missing[field].append(dependency)
        if missing:
            raise serializers.ValidationError(self.get_message(values, missing=missing))



from typing import Any

from rest_framework import serializers


class CSVField(serializers.ListField):
    """
    Parses a comma-separated string into a list of values.

    Requires a ``child`` argument for validation and conversion for each item.
    You can provide ``serializers.ChoiceField`` as the child to simulate a
    multiple-choice field.

    This is a subclass of ``serializers.ListField``, so you can specify
    parameters such as ``min_length``, ``max_length`` and ``allow_empty``.
    """

    def to_internal_value(self, data: Any) -> list[Any]:
        return super().to_internal_value(data.split(","))



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Literal

from django.conf import settings

from rest_framework.settings import api_settings

from rest_filters.utils import notset


def get_default_known_parameters() -> list[str]:
    params = [
        "page",
        "page_size",
        "cursor",
        api_settings.ORDERING_PARAM,
        api_settings.VERSION_PARAM,
    ]
    if format_param := api_settings.URL_FORMAT_OVERRIDE:
        params.append(format_param)
    return params


@dataclass(frozen=True)
class AppSettings:
    """
    There are a few settings that change how ``rest-filters`` behaves globally.
    Most of these settings can also be changed on a per-FilterSet basis.

    You can use these settings by adding the ``REST_FILTERS`` setting to your
    Django configuration file. For example:

    .. code-block:: python

        REST_FILTERS = {
            "BLANK": "keep",
            "KNOWN_PARAMETERS": ["page", "page_size"],
        }
    """

    BLANK: Literal["keep"] | Literal["omit"] = "omit"
    """
    Determines how empty query parameters are handled. Default is ``omit``
    which behaves as if query parameter was not provided. Setting this to
    ``keep`` will cause empty values to be parsed by the related field.
    """
    KNOWN_PARAMETERS: list[str] = notset  # type: ignore[assignment]
    """
    A list of query parameters that are not defined in FilterSet but otherwise
    used by other mechanisms, such as pagination.

    By default, the following query parameters are marked as known:

    - page
    - page_size
    - cursor
    - ``api_settings.ORDERING_PARAM``
    - ``api_settings.VERSION_PARAM``
    - ``api_settings.URL_FORMAT_OVERRIDE``
    """
    HANDLE_UNKNOWN_PARAMETERS: bool = True
    """
    Decides whether to handle unknown parameters.
    """

    def __getattribute__(self, __name: str) -> Any:
        user_settings = getattr(settings, "REST_FILTERS", {})
        value = user_settings.get(__name, super().__getattribute__(__name))
        if value is notset:
            if __name == "KNOWN_PARAMETERS":
                # Can't use this as the default factory since it would access
                # DRF settings at import time.
                return get_default_known_parameters()
        return value


app_settings = AppSettings()



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from django.db.models import QuerySet

from rest_framework import filters
from rest_framework.request import Request
from rest_framework.views import APIView

from rest_filters.utils import _get_filterset_schema, _MT_co

if TYPE_CHECKING:
    from rest_filters import FilterSet

__all__ = [
    "FilterBackend",
]


class FilterBackend(filters.BaseFilterBackend):
    def _get_filterset_class(self, view: APIView) -> type[FilterSet[Any]] | None:
        if klass := getattr(view, "filterset_class", None):
            return klass  # type: ignore[no-any-return]
        try:
            return view.get_filterset_class()  # type: ignore[no-any-return, attr-defined]
        except AttributeError:
            return None

    def get_filterset_class(
        self,
        request: Request,
        queryset: QuerySet[_MT_co],
        view: APIView,
    ) -> type[FilterSet[_MT_co]] | None:
        return self._get_filterset_class(view)

    def get_filterset(
        self,
        request: Request,
        queryset: QuerySet[_MT_co],
        view: APIView,
    ) -> FilterSet[_MT_co] | None:
        klass = self.get_filterset_class(request, queryset, view)
        if klass is None:
            return None
        return klass(request, queryset, view)

    def filter_queryset(
        self,
        request: Request,
        queryset: QuerySet[_MT_co],
        view: APIView,
    ) -> QuerySet[_MT_co]:
        filterset = self.get_filterset(request, queryset, view)
        if filterset is None:
            return queryset
        return filterset.filter_queryset()

    def get_schema_operation_parameters(self, view: APIView) -> list[dict[str, Any]]:
        filterset = self._get_filterset_class(view)
        if filterset is None:
            return []
        return _get_filterset_schema(filterset=filterset, view=view)



from __future__ import annotations

import enum
import warnings
from typing import TYPE_CHECKING, Any, Final, TypeVar

from django.db import models
from django.db.models import Q

from rest_framework import serializers
from rest_framework.fields import Field

_MT_co = TypeVar("_MT_co", bound=models.Model, covariant=True)

NotSet = enum.Enum("NotSet", "notset")
notset: Final = NotSet.notset

if TYPE_CHECKING:
    from rest_framework.views import APIView

    from rest_filters import Filter, FilterSet

    AnyField = Field[Any, Any, Any, Any]
else:
    AnyField = Field


def fill_q_template(template: Q, *, value: Any) -> Q:
    conditions: list[Any] = []
    for child in template.children:
        if isinstance(child, Q):
            conditions.append(fill_q_template(child, value=value))
        else:
            if isinstance(child, tuple):
                field, value = child
                raise ValueError(
                    "Q objects should not specify values in templates, got"
                    " Q(%(field)s=%(value)r), expected Q(%(field)r)."
                    " If you would like to do more complex queries, use `method`"
                    " argument." % {"field": field, "value": value}
                )
            conditions.append((child, value))
    return Q(
        *conditions,
        _connector=template.connector,
        _negated=template.negated,
    )


def merge_errors(
    source: dict[str, Any],
    errors: dict[str, Any],
) -> None:
    for key, detail in errors.items():
        src = source.get(key)
        if isinstance(src, list):
            if isinstance(detail, (list, tuple)):
                source[key].extend(detail)
            else:
                source[key].append(detail)
        elif isinstance(src, dict) and isinstance(detail, dict):
            merge_errors(source[key], detail)
        else:
            source.setdefault(key, detail)


def _filter_to_schema(
    f: Filter,
    /,
    *,
    view: APIView,
) -> dict[str, Any] | None:
    try:
        field = f.get_serializer()
    except ValueError:
        warnings.warn(
            "Could not determine schema for query parameter: %s" % f.get_param_name(),
            stacklevel=1,
        )
        return None
    if isinstance(field, serializers.Serializer):
        schema = view.schema._map_serializer(field, "request")  # type: ignore[union-attr]
    else:
        schema = view.schema._map_serializer_field(field, "request")  # type: ignore[union-attr]
    return {
        "name": f.get_param_name(),
        "in": "query",
        "required": field.required,
        "schema": schema,
        "explode": False,
    }


def _get_filterset_schema(
    *,
    filterset: type[FilterSet[Any]],
    view: APIView,
) -> list[dict[str, Any]]:
    ret = []
    for _, field in filterset.compiled_fields.items():
        for f in [field, *field.get_all_children()]:
            if f.namespace:
                continue
            schema = _filter_to_schema(f, view=view)
            if schema is not None:
                ret.append(schema)
    return ret



from __future__ import annotations

import copy
import functools
import itertools
import operator
from collections import defaultdict
from collections.abc import Sequence
from difflib import get_close_matches
from typing import TYPE_CHECKING, Any, Generic, Literal, final

from django.db.models import QuerySet
from django.utils.translation import gettext

from rest_framework import serializers
from rest_framework.fields import empty
from rest_framework.request import Request
from rest_framework.settings import api_settings
from rest_framework.views import APIView

from rest_filters.conf import app_settings
from rest_filters.constraints import Constraint
from rest_filters.filters import Entry, Filter
from rest_filters.utils import AnyField, NotSet, _MT_co, merge_errors, notset

if TYPE_CHECKING:
    from rest_framework.fields import _Empty


@final
class Options:
    __slots__ = (
        "_blank",
        "_extend_known_parameters",
        "_handle_unknown_parameters",
        "_known_parameters",
        "combinators",
        "constraints",
        "fields",
    )

    def __init__(
        self,
        *,
        fields: list[str] | tuple[str] | NotSet = notset,
        known_parameters: list[str] | tuple[str] | NotSet = notset,
        extend_known_parameters: list[str] | tuple[str] | NotSet = notset,
        handle_unknown_parameters: bool | NotSet = notset,
        constraints: Sequence[Constraint] | NotSet = notset,
        combinators: dict[str, Any] | NotSet = notset,
        blank: Literal["keep", "omit"] | NotSet = notset,
    ) -> None:
        """
        The following parameters can be used as class attributes in
        ``FilterSet.Meta``:

        :param fields: A subset of available query parameters that will be used
         in this FilterSet. Use this to disable certain query parameters.
        :param known_parameters: Overrides
         :py:attr:`rest_filters.conf.AppSettings.KNOWN_PARAMETERS`
        :param handle_unknown_parameters: Overrides
         :py:attr:`rest_filters.conf.AppSettings.HANDLE_UNKNOWN_PARAMETERS`
        :param constraints: A list of constraint instances that are going to be
         enforced for this FilterSet.
        :param combinators: A dictionary that contains the query combination
         operator for given groups. The default operator for groups is
         ``operator.and_``.
        :param blank: Overrides :py:attr:`rest_filters.conf.AppSettings.BLANK`
        """
        self._known_parameters = known_parameters
        self._extend_known_parameters = extend_known_parameters
        self._handle_unknown_parameters = handle_unknown_parameters
        self._blank = blank

        if constraints is notset:
            constraints = []
        if combinators is notset:
            combinators = {}

        self.fields = fields
        self.constraints = constraints
        self.combinators = combinators

    @property
    def known_parameters(self) -> tuple[str] | list[str]:
        params = self._known_parameters
        if params is notset:
            params = app_settings.KNOWN_PARAMETERS
        if self._extend_known_parameters is not notset:
            params = (*params, *self._extend_known_parameters)
        return params

    @property
    def handle_unknown_parameters(self) -> bool:
        if self._handle_unknown_parameters is notset:
            return app_settings.HANDLE_UNKNOWN_PARAMETERS
        return self._handle_unknown_parameters

    @property
    def blank(self) -> Literal["keep", "omit"]:
        if self._blank is notset:
            return app_settings.BLANK
        return self._blank


class FilterSet(Generic[_MT_co]):
    options: Options
    compiled_fields: dict[str, Filter]

    def __init__(
        self,
        request: Request,
        queryset: QuerySet[_MT_co],
        view: APIView,
    ) -> None:
        self.request = request
        """Django REST framework ``Request`` object."""
        self.queryset = queryset
        self.view = view
        """View instance for this request."""

        self._fields = copy.deepcopy(self.compiled_fields)
        self._constraints = copy.deepcopy(self.options.constraints)

    def __init_subclass__(cls, **kwargs: Any) -> None:
        meta_fields = (
            "fields",
            "constraints",
            "combinators",
            "known_parameters",
            "extend_known_parameters",
            "handle_unknown_parameters",
            "blank",
        )
        if meta := getattr(cls, "Meta", None):
            opts = {field: getattr(meta, field, notset) for field in meta_fields}
            options = Options(**opts)
        else:
            options = Options()
        cls.options = options
        cls.compiled_fields = cls._compile_fields()

    @classmethod
    def _visit(
        cls, fields: Sequence[str], f: Filter
    ) -> tuple[list[str], Filter | None]:
        param = f.get_param_name()
        keep = param in fields
        if not f.children:
            if keep:
                return [param], f
            return [param], None
        visits = [cls._visit(fields, child) for child in f.children]
        p, c = zip(*visits, strict=True)
        params, children = (
            list(itertools.chain.from_iterable(p)),
            [child for child in c if child is not None],
        )
        if children:
            f.children = children
            if not keep:
                f.namespace = True
        elif keep:
            f.children = []
        else:
            return params, None
        params.append(param)
        return params, f

    @classmethod
    def _compile_fields(cls) -> dict[str, Filter]:
        fields = {
            name: field
            for name, field in vars(cls).items()
            if isinstance(field, Filter)
        }
        if cls.options.fields is notset:
            return fields
        ret, available = {}, []
        for name, field in fields.items():
            params, f = cls._visit(cls.options.fields, field)
            available.extend(params)
            if f is not None:
                ret[name] = f
        unknown = [field for field in cls.options.fields if field not in available]
        if unknown:
            raise ValueError(
                "The following fields are not valid: %(fields)s,"
                " available fields: %(available)s"
                % {
                    "fields": ", ".join((repr(item) for item in unknown)),
                    "available": ", ".join(repr(item) for item in available),
                }
            )
        return ret

    def get_groups(self) -> tuple[dict[str, dict[str, Entry]], dict[str, Any]]:
        params = self.request.query_params
        fields = self.get_fields()
        groupdict: dict[str, dict[str, Entry]]
        groupdict, valuedict, errordict = defaultdict(dict), {}, {}
        known = [*self.options.known_parameters]
        for _, field in fields.items():
            try:
                field._filterset = self
                entries, errors = field.resolve(params)
            finally:
                field._filterset = None
            known.extend((*entries, *errors))
            for param, entry in entries.items():
                if entry is not None:
                    groupdict[entry.group][param] = entry
                    valuedict[param] = entry.value
            for param, error in errors.items():
                errordict[param] = error
                valuedict[param] = empty
        merge_errors(errordict, self.handle_constraints(valuedict))
        if self.options.handle_unknown_parameters:
            unknown = [field for field in params if field not in known]
            if unknown:
                merge_errors(errordict, self.handle_unknown_parameters(unknown, known))
        if errordict:
            self.handle_errors(errordict)
        return dict(groupdict), valuedict

    def add_to_queryset(
        self, queryset: QuerySet[_MT_co], entry: Entry
    ) -> QuerySet[_MT_co]:
        if entry.expression is None:
            return queryset
        if entry.aliases:
            queryset = queryset.alias(**entry.aliases)
        return queryset.filter(entry.expression)

    def filter_group(
        self,
        queryset: QuerySet[_MT_co],
        group: str,
        entries: dict[str, Entry],
    ) -> QuerySet[_MT_co]:
        return self.add_to_queryset(queryset, self.get_group_entry(group, entries))

    def get_group_entry(self, group: str, entries: dict[str, Entry]) -> Entry:
        """
        Resolve Entry for the given group.

        :param group: Name of the group that is currently being resolved.
        :param entries: Query parameters belonging to this group, with their
         corresponding Entry.
        """
        combinator = self.options.combinators.get(group, operator.and_)
        expressions = [
            entry.expression
            for entry in entries.values()
            if entry.expression is not None
        ]
        if expressions:
            expression = functools.reduce(combinator, expressions)
        else:
            expression = None
        return Entry(
            group=group,
            aliases=functools.reduce(
                operator.or_,
                (
                    entry.aliases
                    for entry in entries.values()
                    if entry.aliases is not None
                ),
                {},
            )
            or None,
            value={name: entry.value for name, entry in entries.items()},
            expression=expression,
        )

    def filter_queryset(self) -> QuerySet[_MT_co]:
        queryset = self.queryset
        groupdict, valuedict = self.get_groups()

        for entry in groupdict.pop("chain", {}).values():
            queryset = self.add_to_queryset(queryset, entry)

        for name, entries in groupdict.items():
            queryset = self.filter_group(queryset, name, entries)
        return self.get_queryset(queryset, valuedict)

    def get_queryset(
        self,
        queryset: QuerySet[_MT_co],
        values: dict[str, Any],
    ) -> QuerySet[_MT_co]:
        """
        Returns the final QuerySet object. At this point, all the filters are
        applied. Override this method to perform operations on QuerySet that
        are otherwise not possible, such as ``order_by()`` and ``distinct()``
        calls.

        :param queryset: Filtered QuerySet object.
        :param values: Parsed query parameters.
        """
        return queryset

    def get_fields(self) -> dict[str, Filter]:
        """
        Resolve filters that are going to be used in this FilterSet. You may
        override this method to dynamically add filters.

        .. danger::

            Make sure additional Filter instances are initialized inside this
            method, using global variables will lead to dangling references.
        """
        return self._fields

    def get_default(self, param: str, default: Any) -> Any:
        """
        Dynamically determine the default value for the given param.

        :param param: Parameter name.
        :param default: Default value that is otherwise going to be used.
        :return: Default value.
        """
        return default

    def get_serializer(self, param: str, serializer: AnyField | None) -> AnyField:
        """
        Dynamically resolve the serializer field for the given param.

        :param param: Parameter name.
        :param serializer: Serializer field that is otherwise going to be used.
        :return: Serializer field.
        """
        return serializer  # type: ignore[return-value]

    def get_serializer_context(self, param: str) -> dict[str, Any]:
        """
        Get serializer context for the given param. By default, this will use
        ``view.get_serializer_context()``. The context will also include this
        FilterSet instance.

        :param param: Parameter name.
        :return: Context dictionary.
        """
        context: dict[str, Any] = self.view.get_serializer_context()  # type: ignore[attr-defined]
        context["filterset"] = self
        return context

    def run_validation(
        self, value: str | _Empty, serializer: AnyField, param: str
    ) -> Any:
        """
        Run validation for the given param.

        :param value: Value provided by the user. This will be ``empty`` if the
         parameter is missing.
        :param serializer: Serializer field that is going to be used for
         validation.
        :param param: Parameter name.
        :return: Parsed query parameter value.
        """
        return serializer.run_validation(value)

    def get_constraints(self) -> Sequence[Constraint]:
        """
        Resolve constraint objects that are going to be used in this FilterSet.
        You may override this method to dynamically add constraints.

        .. danger::

            Make sure additional Constraint instances are initialized inside
            this method, using global variables will lead to dangling
            references.
        """
        return self._constraints

    def handle_constraints(self, valuedict: dict[str, Any]) -> dict[str, Any]:
        errors: dict[str, Any] = {}
        constraints = self.get_constraints()
        for constraint in constraints:
            constraint.filterset = self
            try:
                constraint.check(valuedict)
            except serializers.ValidationError as err:
                detail = err.detail
                if not isinstance(detail, dict):
                    detail = {api_settings.NON_FIELD_ERRORS_KEY: detail}
                merge_errors(errors, detail)
            finally:
                constraint.filterset = None
        return errors

    def handle_unknown_parameters(
        self, unknown: list[str], known: list[str]
    ) -> dict[str, Any]:
        """
        Creates error messages for unknown parameters.

        :param unknown: Unknown parameters the user supplied.
        :param known: Known parameters.
        :return: An error dictionary.
        """
        fields = {}
        for param in unknown:
            matches = get_close_matches(param, known)
            if not matches:
                fields[param] = [gettext("This query parameter does not exist.")]
            elif len(matches) == 1:
                fields[param] = [
                    gettext(
                        'This query parameter does not exist. Did you mean "%(param)s"?'
                    )
                    % {"param": matches[0]}
                ]
            else:
                possibilities = ", ".join(f'"{match}"' for match in matches)
                fields[param] = [
                    gettext(
                        "This query parameter does not exist."
                        " Did you mean one of these: %(possibilities)s?"
                    )
                    % {"possibilities": possibilities}
                ]
        return fields

    def handle_errors(self, errordict: dict[str, Any]) -> None:
        """
        Raises ``ValidationError`` for given errors. You may override this
        method to change the error format.
        """
        raise serializers.ValidationError(errordict)



from rest_filters.backends import FilterBackend
from rest_filters.filters import Filter
from rest_filters.filtersets import FilterSet

__all__ = [
    "Filter",
    "FilterBackend",
    "FilterSet",
]
